@@ -24,7 +24,7 @@ import ldap
 from ldap.controls import SimplePagedResultsControl
 from ldap.filter import escape_filter_chars
 import time
-ldap_pagination_size = 1000
+ldap_pagination_size = None
 ldap_timeout = None
 
 logging.getLogger('requests').setLevel(logging.WARNING)
@@ -414,7 +414,16 @@ def get_ldap_lists(l, search_strings, parent_list=None):
                              serverctrls=[req_ctrl],
                              attrlist=(search_strings['list_cn_field'], search_strings['list_name_field']),
                              filterstr=filterstr)
-        rtype, rdata, rmsgid, serverctrls = l.result3(msgid, timeout=ldap_timeout, resp_ctrl_classes=known_ldap_resp_ctrls)
+
+        retries = 0
+        while retries < 5:
+            try:
+                rtype, rdata, rmsgid, serverctrls = l.result3(msgid, timeout=ldap_timeout, resp_ctrl_classes=known_ldap_resp_ctrls)
+            except Exception:
+                logger.exception('Error getting ldap list memberships')
+                retries += 1
+            else:
+                break
 
         for (dn, data) in rdata:
             cn_field = data[search_strings['list_cn_field']][0]
@@ -432,6 +441,7 @@ def get_ldap_lists(l, search_strings, parent_list=None):
 
         if not pctrls:
             # Paging not supported
+            logger.debug('LDAP pagination not supported')
             break
         cookie = pctrls[0].cookie
         if not cookie:
@@ -454,7 +464,15 @@ def get_ldap_list_membership(l, search_strings, list_name):
                              attrlist=[search_strings['user_mail_field']],
                              filterstr=search_strings['user_membership_filter'] % escape_filter_chars(list_name)
                              )
-        rtype, rdata, rmsgid, serverctrls = l.result3(msgid, timeout=ldap_timeout, resp_ctrl_classes=known_ldap_resp_ctrls)
+        retries = 0
+        while retries < 5:
+            try:
+                rtype, rdata, rmsgid, serverctrls = l.result3(msgid, timeout=ldap_timeout, resp_ctrl_classes=known_ldap_resp_ctrls)
+            except Exception:
+                logger.exception('Error getting ldap list memberships')
+                retries += 1
+            else:
+                break
 
         for data in rdata:
             member = data[1].get(search_strings['user_mail_field'], [None])[0]
@@ -467,6 +485,7 @@ def get_ldap_list_membership(l, search_strings, list_name):
 
         if not pctrls:
             # Paging not supported
+            logger.debug('LDAP pagination not supported')
             break
         cookie = pctrls[0].cookie
         if not cookie:
@@ -665,13 +684,16 @@ def sync_ldap_lists(ldap_settings, engine):
 
 def main():
     global ldap_timeout
+    global ldap_pagination_size
     config = load_config()
     metrics.init(config, 'iris-sync-targets', stats_reset)
 
     default_ldap_timeout = 60
+    default_ldap_pagination_size = 400
     default_nap_time = 3600
 
     ldap_timeout = int(config.get('sync_script_ldap_timeout', default_ldap_timeout))
+    ldap_pagination_size = int(config.get('sync_script_ldap_pagination_size', default_ldap_pagination_size))
     try:
         nap_time = int(config.get('sync_script_nap_time', default_nap_time))
     except ValueError:
